from yattag import Doc

from .ReferenceMethod import ReferenceMethod
from .SimpleContent import (
    PreparationEndDate,
    PreparationStartDate,
    SubstanceDilutionFactor,
)
from .WQXTime import WQXTime


class LabSamplePreparation:
    """
    Describes Lab Sample Preparation procedures which may alter the original state of
    the Sample and produce Lab subsamples.  These Lab Subsamples are analyized and
    reported by the Lab as Sample results.
    """

    __labSamplePreparationMethod: ReferenceMethod
    __preparationStartDate: PreparationStartDate
    __preparationStartTime: WQXTime
    __preparationEndDate: PreparationEndDate
    __preparationEndTime: WQXTime
    __substanceDilutionFactor: SubstanceDilutionFactor

    def __init__(
        self,
        o: dict = None,
        *,
        labSamplePreparationMethod: ReferenceMethod = None,
        preparationStartDate: PreparationStartDate = None,
        preparationStartTime: WQXTime = None,
        preparationEndDate: PreparationEndDate = None,
        preparationEndTime: WQXTime = None,
        substanceDilutionFactor: SubstanceDilutionFactor = None
    ):
        if isinstance(o, LabSamplePreparation):
            # Assign attributes from object without typechecking
            self.__labSamplePreparationMethod = o.labSamplePreparationMethod
            self.__preparationStartDate = o.preparationStartDate
            self.__preparationStartTime = o.preparationStartTime
            self.__preparationEndDate = o.preparationEndDate
            self.__preparationEndTime = o.preparationEndTime
            self.__substanceDilutionFactor = o.substanceDilutionFactor
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.labSamplePreparationMethod = o.get("labSamplePreparationMethod")
            self.preparationStartDate = o.get("preparationStartDate")
            self.preparationStartTime = o.get("preparationStartTime")
            self.preparationEndDate = o.get("preparationEndDate")
            self.preparationEndTime = o.get("preparationEndTime")
            self.substanceDilutionFactor = o.get("substanceDilutionFactor")
        else:
            # Assign attributes from named keywords with typechecking
            self.labSamplePreparationMethod = labSamplePreparationMethod
            self.preparationStartDate = preparationStartDate
            self.preparationStartTime = preparationStartTime
            self.preparationEndDate = preparationEndDate
            self.preparationEndTime = preparationEndTime
            self.substanceDilutionFactor = substanceDilutionFactor

    @property
    def labSamplePreparationMethod(self) -> ReferenceMethod:
        """
        Identifying information about the method followed to prepare a sample for
        analysis.
        """
        return self.__labSamplePreparationMethod

    @labSamplePreparationMethod.setter
    def labSamplePreparationMethod(self, val: ReferenceMethod) -> None:
        """
        Identifying information about the method followed to prepare a sample for
        analysis.
        """
        self.__labSamplePreparationMethod = None if val is None else ReferenceMethod(val)

    @property
    def preparationStartDate(self) -> PreparationStartDate:
        return self.__preparationStartDate

    @preparationStartDate.setter
    def preparationStartDate(self, val: PreparationStartDate) -> None:
        self.__preparationStartDate = None if val is None else PreparationStartDate(val)

    @property
    def preparationStartTime(self) -> WQXTime:
        """
        The local time when the preparation/extraction of the sample for analysis began.
        """
        return self.__preparationStartTime

    @preparationStartTime.setter
    def preparationStartTime(self, val: WQXTime) -> None:
        """
        The local time when the preparation/extraction of the sample for analysis began.
        """
        self.__preparationStartTime = None if val is None else WQXTime(val)

    @property
    def preparationEndDate(self) -> PreparationEndDate:
        return self.__preparationEndDate

    @preparationEndDate.setter
    def preparationEndDate(self, val: PreparationEndDate) -> None:
        self.__preparationEndDate = None if val is None else PreparationEndDate(val)

    @property
    def preparationEndTime(self) -> WQXTime:
        """
        The local time when the preparation/extraction of the sample for analysis was
        finished.
        """
        return self.__preparationEndTime

    @preparationEndTime.setter
    def preparationEndTime(self, val: WQXTime) -> None:
        """
        The local time when the preparation/extraction of the sample for analysis was
        finished.
        """
        self.__preparationEndTime = None if val is None else WQXTime(val)

    @property
    def substanceDilutionFactor(self) -> SubstanceDilutionFactor:
        return self.__substanceDilutionFactor

    @substanceDilutionFactor.setter
    def substanceDilutionFactor(self, val: SubstanceDilutionFactor) -> None:
        self.__substanceDilutionFactor = (
            None if val is None else SubstanceDilutionFactor(val)
        )

    def generateXML(self, name: str = "LabSamplePreparation") -> str:
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__labSamplePreparationMethod is not None:
                asis(
                    self.__labSamplePreparationMethod.generateXML(
                        "LabSamplePreparationMethod"
                    )
                )
            if self.__preparationStartDate is not None:
                line("PreparationStartDate", self.__preparationStartDate)
            if self.__preparationStartTime is not None:
                asis(self.__preparationStartTime.generateXML("PreparationStartTime"))
            if self.__preparationEndDate is not None:
                line("PreparationEndDate", self.__preparationEndDate)
            if self.__preparationEndTime is not None:
                asis(self.__preparationEndTime.generateXML("PreparationEndTime"))
            if self.__substanceDilutionFactor is not None:
                line("SubstanceDilutionFactor", self.__substanceDilutionFactor)

        return doc.getvalue()



from .wqx_v3_0 import AttachedBinaryObject


class WQXAttachedBinaryObject(AttachedBinaryObject):
    def __init__(self,):
        AttachedBinaryObject.__init__(self)

    def __enter__(self):
        return self

    def __exit__(self, exception_type, exception_value, exception_traceback):
        pass  # TODO: add a rule test



from yattag import Doc

from ..exceptions import WQXException
from .SimpleContent import (
    OrganizationDescriptionText,
    OrganizationFormalName,
    OrganizationIdentifier,
    TribalCode,
)


class OrganizationDescription:
    """
    The particular word(s) regularly connected with a unique framework of authority
    within which a person or persons act, or are designated to act, towards some purpose.
    """

    __organizationIdentifier: OrganizationIdentifier = None
    __organizationFormalName: OrganizationFormalName = None
    __organizationDescriptionText: OrganizationDescriptionText = None
    __tribalCode: TribalCode = None

    def __init__(
        self,
        o: dict = None,
        *,
        organizationIdentifier: OrganizationIdentifier = None,
        organizationFormalName: OrganizationFormalName = None,
        organizationDescriptionText: OrganizationDescriptionText = None,
        tribalCode: TribalCode = None
    ):
        if isinstance(o, OrganizationDescription):
            # Assign attributes from object without typechecking
            self.__organizationIdentifier = o.organizationIdentifier
            self.__organizationFormalName = o.organizationFormalName
            self.__organizationDescriptionText = o.organizationDescriptionText
            self.__tribalCode = o.tribalCode
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.organizationIdentifier = o.get("organizationIdentifier")
            self.organizationFormalName = o.get("organizationFormalName")
            self.organizationDescriptionText = o.get("organizationDescriptionText")
            self.tribalCode = o.get("tribalCode")
        else:
            # Assign attributes from named keywords with typechecking
            self.organizationIdentifier = organizationIdentifier
            self.organizationFormalName = organizationFormalName
            self.organizationDescriptionText = organizationDescriptionText
            self.tribalCode = tribalCode

    @property
    def organizationIdentifier(self) -> OrganizationIdentifier:
        return self.__organizationIdentifier

    @organizationIdentifier.setter
    def organizationIdentifier(self, val: OrganizationIdentifier) -> None:
        self.__organizationIdentifier = (
            None if val is None else OrganizationIdentifier(val)
        )

    @property
    def organizationFormalName(self) -> OrganizationFormalName:
        return self.__organizationFormalName

    @organizationFormalName.setter
    def organizationFormalName(self, val: OrganizationFormalName) -> None:
        self.__organizationFormalName = (
            None if val is None else OrganizationFormalName(val)
        )

    @property
    def organizationDescriptionText(self) -> OrganizationDescriptionText:
        return self.__organizationDescriptionText

    @organizationDescriptionText.setter
    def organizationDescriptionText(self, val: OrganizationDescriptionText) -> None:
        self.__organizationDescriptionText = (
            None if val is None else OrganizationDescriptionText(val)
        )

    @property
    def tribalCode(self) -> TribalCode:
        return self.__tribalCode

    @tribalCode.setter
    def tribalCode(self, val: TribalCode) -> None:
        self.__tribalCode = None if val is None else TribalCode(val)

    def generateXML(self, name: str = "OrganizationDescription") -> str:
        doc = Doc()
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.organizationIdentifier is None:
                raise WQXException("Attribute 'organizationIdentifier' is required.")
            line("OrganizationIdentifier", self.__organizationIdentifier)
            if self.organizationFormalName is None:
                raise WQXException("Attribute 'organizationFormalName' is required.")
            line("OrganizationFormalName", self.__organizationFormalName)
            if self.__organizationDescriptionText is not None:
                line("OrganizationDescriptionText", self.__organizationDescriptionText)
            if self.__tribalCode is not None:
                line("TribalCode", self.__tribalCode)

        return doc.getvalue()



from yattag import Doc

from ..exceptions import WQXException
from .SimpleContent import (
    ResourceCreatorName,
    ResourceDate,
    ResourceIdentifier,
    ResourcePublisherName,
    ResourceSubjectText,
    ResourceTitleName,
)


class BibliographicReference:
    """
    The descriptors used to identify and catalog an object.
    """

    __resourceTitleName: ResourceTitleName
    __resourceCreatorName: ResourceCreatorName
    __resourceSubjectText: ResourceSubjectText
    __resourcePublisherName: ResourcePublisherName
    __resourceDate: ResourceDate
    __resourceIdentifier: ResourceIdentifier

    def __init__(
        self,
        o: dict = None,
        *,
        resourceTitleName: ResourceTitleName = None,
        resourceCreatorName: ResourceCreatorName = None,
        resourceSubjectText: ResourceSubjectText = None,
        resourcePublisherName: ResourcePublisherName = None,
        resourceDate: ResourceDate = None,
        resourceIdentifier: ResourceIdentifier = None
    ):
        if isinstance(o, BibliographicReference):
            # Assign attributes from object without typechecking
            self.__resourceTitleName = resourceTitleName
            self.__resourceCreatorName = resourceCreatorName
            self.__resourceSubjectText = resourceSubjectText
            self.__resourcePublisherName = resourcePublisherName
            self.__resourceDate = resourceDate
            self.__resourceIdentifier = resourceIdentifier
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.resourceTitleName = o.get("resourceTitleName")
            self.resourceCreatorName = o.get("resourceCreatorName")
            self.resourceSubjectText = o.get("resourceSubjectText")
            self.resourcePublisherName = o.get("resourcePublisherName")
            self.resourceDate = o.get("resourceDate")
            self.resourceIdentifier = o.get("resourceIdentifier")
        else:
            # Assign attributes from named keywords with typechecking
            self.resourceTitleName = resourceTitleName
            self.resourceCreatorName = resourceCreatorName
            self.resourceSubjectText = resourceSubjectText
            self.resourcePublisherName = resourcePublisherName
            self.resourceDate = resourceDate
            self.resourceIdentifier = resourceIdentifier

    @property
    def ResourceTitleName(self) -> ResourceTitleName:
        return self.__ResourceTitleName

    @ResourceTitleName.setter
    def ResourceTitleName(self, val: ResourceTitleName) -> None:
        self.__ResourceTitleName = ResourceTitleName(val)

    @property
    def ResourceCreatorName(self) -> ResourceCreatorName:
        return self.__ResourceCreatorName

    @ResourceCreatorName.setter
    def ResourceCreatorName(self, val: ResourceCreatorName) -> None:
        self.__ResourceCreatorName = None if val is None else ResourceCreatorName(val)

    @property
    def ResourceSubjectText(self) -> ResourceSubjectText:
        return self.__ResourceSubjectText

    @ResourceSubjectText.setter
    def ResourceSubjectText(self, val: ResourceSubjectText) -> None:
        self.__ResourceSubjectText = None if val is None else ResourceSubjectText(val)

    @property
    def ResourcePublisherName(self) -> ResourcePublisherName:
        return self.__ResourcePublisherName

    @ResourcePublisherName.setter
    def ResourcePublisherName(self, val: ResourcePublisherName) -> None:
        self.__ResourcePublisherName = None if val is None else ResourcePublisherName(val)

    @property
    def ResourceDate(self) -> ResourceDate:
        return self.__ResourceDate

    @ResourceDate.setter
    def ResourceDate(self, val: ResourceDate) -> None:
        self.__ResourceDate = ResourceDate(val)

    @property
    def ResourceIdentifier(self) -> ResourceIdentifier:
        return self.__ResourceIdentifier

    @ResourceIdentifier.setter
    def ResourceIdentifier(self, val: ResourceIdentifier) -> None:
        self.__ResourceIdentifier = ResourceIdentifier(val)

    def generateXML(self, name: str = "BibliographicReference") -> str:
        doc = Doc()
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__resourceTitleName is None:
                raise WQXException("Attribute 'ResourceTitleName' is required.")
            line("ResourceTitleName", self.__resourceTitleName)
            if self.__resourceCreatorName is not None:
                line("ResourceCreatorName", self.__resourceCreatorName)
            if self.__resourceSubjectText is not None:
                line("ResourceSubjectText", self.__resourceSubjectText)
            if self.__resourcePublisherName is not None:
                line("ResourcePublisherName", self.__resourcePublisherName)
            if self.__resourceDate is None:
                raise WQXException("Attribute 'ResourceDate' is required.")
            line("ResourceDate", self.__resourceDate)
            if self.__resourceIdentifier is None:
                raise WQXException("Attribute 'ResourceIdentifier' is required.")
            line("ResourceIdentifier", self.__resourceIdentifier)

        return doc.getvalue()



# import setuptools
import runpy

from setuptools import find_packages, setup

version_meta = runpy.run_path("./version.py")
VERSION = version_meta["__version__"]

with open("README.md", "r") as fh:
    long_description = fh.read()


def parse_requirements(filename):
    """
    Load requirements from a pip requirements file.
    """
    lineiter = (line.strip() for line in open(filename))
    return [line for line in lineiter if line and not line.startswith("#")]


setup(
    name="wqxlib",
    version=VERSION,
    author="Jon Musselwhite",
    author_email="JMusselwhite@wvstateu.edu",
    description="A library to create files for the EPA's WQX service.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/FlippingBinary/wqxlib/tree/main/wqxlib-python",
    packages=find_packages(),
    install_requires=parse_requirements("requirements.txt"),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Hydrology",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.6",
)



from yattag import Doc

from ..exceptions import WQXException
from .SimpleContent import (
    MethodDescriptionText,
    MethodIdentifier,
    MethodIdentifierContext,
    MethodName,
    MethodQualifierTypeName,
)


class ResultAnalyticalMethod:
    """
    Identifies the procedures, processes, and references required to determine the
    analytical methods used to obtain a result.
    """

    __methodIdentifier: MethodIdentifier
    __methodIdentifierContext: MethodIdentifierContext
    __methodName: MethodName
    __methodQualifierTypeName: MethodQualifierTypeName
    __methodDescriptionText: MethodDescriptionText

    def __init__(
        self,
        o: dict = None,
        *,
        methodIdentifier: MethodIdentifier = None,
        methodIdentifierContext: MethodIdentifierContext = None,
        methodName: MethodName = None,
        methodQualifierTypeName: MethodQualifierTypeName = None,
        methodDescriptionText: MethodDescriptionText = None
    ):
        if isinstance(o, ResultAnalyticalMethod):
            # Assign attributes from object without typechecking
            self.__methodIdentifier = o.methodIdentifier
            self.__methodIdentifierContext = o.methodIdentifierContext
            self.__methodName = o.methodName
            self.__methodQualifierTypeName = o.methodQualifierTypeName
            self.__methodDescriptionText = o.methodDescriptionText
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.methodIdentifier = o.get("methodIdentifier")
            self.methodIdentifierContext = o.get("methodIdentifierContext")
            self.methodName = o.get("methodName")
            self.methodQualifierTypeName = o.get("methodQualifierTypeName")
            self.methodDescriptionText = o.get("methodDescriptionText")
        else:
            # Assign attributes from named keywords with typechecking
            self.methodIdentifier = methodIdentifier
            self.methodIdentifierContext = methodIdentifierContext
            self.methodName = methodName
            self.methodQualifierTypeName = methodQualifierTypeName
            self.methodDescriptionText = methodDescriptionText

    @property
    def methodIdentifier(self) -> MethodIdentifier:
        return self.__methodIdentifier

    @methodIdentifier.setter
    def methodIdentifier(self, val: MethodIdentifier) -> None:
        self.__methodIdentifier = MethodIdentifier(val)

    @property
    def methodIdentifierContext(self) -> MethodIdentifierContext:
        return self.__methodIdentifierContext

    @methodIdentifierContext.setter
    def methodIdentifierContext(self, val: MethodIdentifierContext) -> None:
        self.__methodIdentifierContext = MethodIdentifierContext(val)

    @property
    def methodName(self) -> MethodName:
        return self.__methodName

    @methodName.setter
    def methodName(self, val: MethodName) -> None:
        self.__methodName = None if val is None else MethodName(val)

    @property
    def methodQualifierTypeName(self) -> MethodQualifierTypeName:
        return self.__methodQualifierTypeName

    @methodQualifierTypeName.setter
    def methodQualifierTypeName(self, val: MethodQualifierTypeName) -> None:
        self.__methodQualifierTypeName = (
            None if val is None else MethodQualifierTypeName(val)
        )

    @property
    def methodDescriptionText(self) -> MethodDescriptionText:
        return self.__methodDescriptionText

    @methodDescriptionText.setter
    def methodDescriptionText(self, val: MethodDescriptionText) -> None:
        self.__methodDescriptionText = None if val is None else MethodDescriptionText(val)

    def generateXML(self, name: str = "ResultAnalyticalMethod") -> str:
        doc = Doc()
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__methodIdentifier is None:
                raise WQXException("Attribute 'methodIdentifier' is required.")
            line("MethodIdentifier", self.__methodIdentifier)
            if self.__methodIdentifierContext is None:
                raise WQXException("Attribute 'methodIdentifierContext' is required.")
            line("MethodIdentifierContext", self.__methodIdentifierContext)
            if self.__methodName is not None:
                line("MethodName", self.__methodName)
            if self.__methodQualifierTypeName is not None:
                line("MethodQualifierTypeName", self.__methodQualifierTypeName)
            if self.__methodDescriptionText is not None:
                line("MethodDescriptionText", self.__methodDescriptionText)

        return doc.getvalue()



from typing import List, Union

from yattag import Doc

from ..exceptions import WQXException
from .MeasureCompact import MeasureCompact
from .SimpleContent import (
    ActivityConductingOrganizationText,
    ActivityEndDate,
    ActivityIdentifier,
    ActivityIdentifierUserSupplied,
    ActivityMediaName,
    ActivityMediaSubdivisionName,
    ActivityRelativeDepthName,
    ActivityStartDate,
    ActivityTypeCode,
    CommentText,
    DepthAltitudeReferencePointText,
    MonitoringLocationIdentifier,
    ProjectIdentifier,
    SamplingComponentName,
)
from .WQXTime import WQXTime


class ActivityDescription:
    """
    Basic identification information for an activity conducted within a project.
    """

    __activityIdentifier: ActivityIdentifier
    __activityIdentifierUserSupplied: ActivityIdentifierUserSupplied
    __activityTypeCode: ActivityTypeCode
    __activityMediaName: ActivityMediaName
    __activityMediaSubdivisionName: ActivityMediaSubdivisionName
    __activityStartDate: ActivityStartDate
    __activityStartTime: WQXTime
    __activityEndDate: ActivityEndDate
    __activityEndTime: WQXTime
    __activityRelativeDepthName: ActivityRelativeDepthName
    __activityDepthHeightMeasure: MeasureCompact
    __activityTopDepthHeightMeasure: MeasureCompact
    __activityBottomDepthHeightMeasure: MeasureCompact
    __activityDepthAltitudeReferencePointText: DepthAltitudeReferencePointText
    __projectIdentifier: ProjectIdentifier
    __activityConductingOrganizationText: List[ActivityConductingOrganizationText]
    __monitoringLocationIdentifier: MonitoringLocationIdentifier
    __samplingComponentName: SamplingComponentName
    __activityCommentText: CommentText

    def __init__(
        self,
        o: dict = None,
        *,
        activityIdentifier: ActivityIdentifier = None,
        activityIdentifierUserSupplied: ActivityIdentifierUserSupplied = None,
        activityTypeCode: ActivityTypeCode = None,
        activityMediaName: ActivityMediaName = None,
        activityMediaSubdivisionName: ActivityMediaSubdivisionName = None,
        activityStartDate: ActivityStartDate = None,
        activityStartTime: WQXTime = None,
        activityEndDate: ActivityEndDate = None,
        activityEndTime: WQXTime = None,
        activityRelativeDepthName: ActivityRelativeDepthName = None,
        activityDepthHeightMeasure: MeasureCompact = None,
        activityTopDepthHeightMeasure: MeasureCompact = None,
        activityBottomDepthHeightMeasure: MeasureCompact = None,
        activityDepthAltitudeReferencePointText: DepthAltitudeReferencePointText = None,
        projectIdentifier: ProjectIdentifier = None,
        activityConductingOrganizationText: List[
            ActivityConductingOrganizationText
        ] = None,
        monitoringLocationIdentifier: MonitoringLocationIdentifier = None,
        samplingComponentName: SamplingComponentName = None,
        activityCommentText: CommentText = None
    ):
        if isinstance(o, ActivityDescription):
            # Assign attributes from object without typechecking
            self.__activityIdentifier = o.activityIdentifier
            self.__activityIdentifierUserSupplied = o.activityIdentifierUserSupplied
            self.__activityTypeCode = o.activityTypeCode
            self.__activityMediaName = o.activityMediaName
            self.__activityMediaSubdivisionName = o.activityMediaSubdivisionName
            self.__activityStartDate = o.activityStartDate
            self.__activityStartTime = o.activityStartTime
            self.__activityEndDate = o.activityEndDate
            self.__activityEndTime = o.activityEndTime
            self.__activityRelativeDepthName = o.activityRelativeDepthName
            self.__activityDepthHeightMeasure = o.activityDepthHeightMeasure
            self.__activityTopDepthHeightMeasure = o.activityTopDepthHeightMeasure
            self.__activityBottomDepthHeightMeasure = o.activityBottomDepthHeightMeasure
            self.__activityDepthAltitudeReferencePointText = (
                o.activityDepthAltitudeReferencePointText
            )
            self.__projectIdentifier = o.projectIdentifier
            self.__activityConductingOrganizationText = (
                o.activityConductingOrganizationText
            )
            self.__monitoringLocationIdentifier = o.monitoringLocationIdentifier
            self.__samplingComponentName = o.samplingComponentName
            self.__activityCommentText = o.activityCommentText
        elif isinstance(o, dict):
            # Assign attributes from other ActivityDescription with typechecking
            self.activityIdentifier = o.get("activityIdentifier")
            self.activityIdentifierUserSupplied = o.get("activityIdentifierUserSupplied")
            self.activityTypeCode = o.get("activityTypeCode")
            self.activityMediaName = o.get("activityMediaName")
            self.activityMediaSubdivisionName = o.get("activityMediaSubdivisionName")
            self.activityStartDate = o.get("activityStartDate")
            self.activityStartTime = o.get("activityStartTime")
            self.activityEndDate = o.get("activityEndDate")
            self.activityEndTime = o.get("activityEndTime")
            self.activityRelativeDepthName = o.get("activityRelativeDepthName")
            self.activityDepthHeightMeasure = o.get("activityDepthHeightMeasure")
            self.activityTopDepthHeightMeasure = o.get("activityTopDepthHeightMeasure")
            self.activityBottomDepthHeightMeasure = o.get(
                "activityBottomDepthHeightMeasure"
            )
            self.activityDepthAltitudeReferencePointText = o.get(
                "activityDepthAltitudeReferencePointText"
            )
            self.projectIdentifier = o.get("projectIdentifier")
            self.activityConductingOrganizationText = o.get(
                "activityConductingOrganizationText", []
            )
            self.monitoringLocationIdentifier = o.get("monitoringLocationIdentifier")
            self.samplingComponentName = o.get("samplingComponentName")
            self.activityCommentText = o.get("activityCommentText")
        else:
            # Assign attributes from named keywords with typechecking
            self.activityIdentifier = activityIdentifier
            self.activityIdentifierUserSupplied = activityIdentifierUserSupplied
            self.activityTypeCode = activityTypeCode
            self.activityMediaName = activityMediaName
            self.activityMediaSubdivisionName = activityMediaSubdivisionName
            self.activityStartDate = activityStartDate
            self.activityStartTime = activityStartTime
            self.activityEndDate = activityEndDate
            self.activityEndTime = activityEndTime
            self.activityRelativeDepthName = activityRelativeDepthName
            self.activityDepthHeightMeasure = activityDepthHeightMeasure
            self.activityTopDepthHeightMeasure = activityTopDepthHeightMeasure
            self.activityBottomDepthHeightMeasure = activityBottomDepthHeightMeasure
            self.activityDepthAltitudeReferencePointText = (
                activityDepthAltitudeReferencePointText
            )
            self.projectIdentifier = projectIdentifier
            self.activityConductingOrganizationText = activityConductingOrganizationText
            self.monitoringLocationIdentifier = monitoringLocationIdentifier
            self.samplingComponentName = samplingComponentName
            self.activityCommentText = activityCommentText

    @property
    def activityIdentifier(self) -> ActivityIdentifier:
        return self.__activityIdentifier

    @activityIdentifier.setter
    def activityIdentifier(self, val: ActivityIdentifier) -> None:
        self.__activityIdentifier = None if val is None else ActivityIdentifier(val)

    @property
    def activityIdentifierUserSupplied(self) -> ActivityIdentifierUserSupplied:
        return self.__activityIdentifierUserSupplied

    @activityIdentifierUserSupplied.setter
    def activityIdentifierUserSupplied(self, val: ActivityIdentifierUserSupplied) -> None:
        self.__activityIdentifierUserSupplied = (
            None if val is None else ActivityIdentifierUserSupplied(val)
        )

    @property
    def activityTypeCode(self) -> ActivityTypeCode:
        return self.__activityTypeCode

    @activityTypeCode.setter
    def activityTypeCode(self, val: ActivityTypeCode) -> None:
        self.__activityTypeCode = None if val is None else ActivityTypeCode(val)

    @property
    def activityMediaName(self) -> ActivityMediaName:
        return self.__activityMediaName

    @activityMediaName.setter
    def activityMediaName(self, val: ActivityMediaName) -> None:
        self.__activityMediaName = None if val is None else ActivityMediaName(val)

    @property
    def activityMediaSubdivisionName(self) -> ActivityMediaSubdivisionName:
        return self.__activityMediaSubdivisionName

    @activityMediaSubdivisionName.setter
    def activityMediaSubdivisionName(self, val: ActivityMediaSubdivisionName) -> None:
        self.__activityMediaSubdivisionName = (
            None if val is None else ActivityMediaSubdivisionName(val)
        )

    @property
    def activityStartDate(self) -> ActivityStartDate:
        return self.__activityStartDate

    @activityStartDate.setter
    def activityStartDate(self, val: ActivityStartDate) -> None:
        self.__activityStartDate = (
            None
            if val is None
            else ActivityStartDate(year=val.year, month=val.month, day=val.day)
        )

    @property
    def activityStartTime(self) -> WQXTime:
        """
        The measure of clock time when the field activity began.
        """
        return self.__activityStartTime

    @activityStartTime.setter
    def activityStartTime(self, val: WQXTime) -> None:
        """
        The measure of clock time when the field activity began.
        """
        self.__activityStartTime = None if val is None else WQXTime(val)

    @property
    def activityEndDate(self) -> ActivityEndDate:
        return self.__activityEndDate

    @activityEndDate.setter
    def activityEndDate(self, val: ActivityEndDate) -> None:
        self.__activityEndDate = (
            None
            if val is None
            else ActivityEndDate(year=val.year, month=val.month, day=val.day)
        )

    @property
    def activityEndTime(self) -> WQXTime:
        """
        The measure of clock time when the field activity ended.
        """
        return self.__activityEndTime

    @activityEndTime.setter
    def activityEndTime(self, val: WQXTime) -> None:
        """
        The measure of clock time when the field activity ended.
        """
        self.__activityEndTime = None if val is None else WQXTime(val)

    @property
    def activityRelativeDepthName(self) -> ActivityRelativeDepthName:
        return self.__activityRelativeDepthName

    @activityRelativeDepthName.setter
    def activityRelativeDepthName(self, val: ActivityRelativeDepthName) -> None:
        self.__activityRelativeDepthName = (
            None if val is None else ActivityRelativeDepthName(val)
        )

    @property
    def activityDepthHeightMeasure(self) -> MeasureCompact:
        """
        A measurement of the vertical location (measured from a reference point) at
        which an activity occurred.
        """
        return self.__activityDepthHeightMeasure

    @activityDepthHeightMeasure.setter
    def activityDepthHeightMeasure(self, val: MeasureCompact) -> None:
        """
        A measurement of the vertical location (measured from a reference point) at
        which an activity occurred.
        """
        self.__activityDepthHeightMeasure = None if val is None else MeasureCompact(val)

    @property
    def activityTopDepthHeightMeasure(self) -> MeasureCompact:
        """
        A measurement of the upper vertical location of a vertical location range
        (measured from a reference point) at which an activity occurred.
        """
        return self.__activityTopDepthHeightMeasure

    @activityTopDepthHeightMeasure.setter
    def activityTopDepthHeightMeasure(self, val: MeasureCompact) -> None:
        """
        A measurement of the upper vertical location of a vertical location range
        (measured from a reference point) at which an activity occurred.
        """
        self.__activityTopDepthHeightMeasure = (
            None if val is None else MeasureCompact(val)
        )

    @property
    def activityBottomDepthHeightMeasure(self) -> MeasureCompact:
        """
        A measurement of the lower vertical location of a vertical location range
        (measured from a reference point) at which an activity occurred.
        """
        return self.__activityBottomDepthHeightMeasure

    @activityBottomDepthHeightMeasure.setter
    def activityBottomDepthHeightMeasure(self, val: MeasureCompact) -> None:
        """
        A measurement of the lower vertical location of a vertical location range
        (measured from a reference point) at which an activity occurred.
        """
        self.__activityBottomDepthHeightMeasure = (
            None if val is None else MeasureCompact(val)
        )

    @property
    def activityDepthAltitudeReferencePointText(self,) -> DepthAltitudeReferencePointText:
        """
        The reference used to indicate the datum or reference used to establish the
        depth/altitude of an activity.
        """
        return self.__activityDepthAltitudeReferencePointText

    @activityDepthAltitudeReferencePointText.setter
    def activityDepthAltitudeReferencePointText(
        self, val: DepthAltitudeReferencePointText
    ) -> None:
        """
        The reference used to indicate the datum or reference used to establish the
        depth/altitude of an activity.
        """
        self.__activityDepthAltitudeReferencePointText = (
            None if val is None else DepthAltitudeReferencePointText(val)
        )

    @property
    def projectIdentifier(self) -> ProjectIdentifier:
        return self.__projectIdentifier

    @projectIdentifier.setter
    def projectIdentifier(self, val: ProjectIdentifier) -> None:
        self.__projectIdentifier = None if val is None else ProjectIdentifier(val)

    @property
    def activityConductingOrganizationText(
        self,
    ) -> List[ActivityConductingOrganizationText]:
        return self.__activityConductingOrganizationText

    @activityConductingOrganizationText.setter
    def activityConductingOrganizationText(
        self,
        val: Union[
            ActivityConductingOrganizationText, List[ActivityConductingOrganizationText]
        ],
    ) -> None:
        if val is None:
            self.__activityConductingOrganizationText = []
        elif isinstance(val, list):
            r: List[ActivityConductingOrganizationText] = []
            for x in val:
                r.append(ActivityConductingOrganizationText(x))
            self.__activityConductingOrganizationText = r
        else:
            self.__activityConductingOrganizationText = [
                ActivityConductingOrganizationText(val)
            ]

    @property
    def monitoringLocationIdentifier(self) -> MonitoringLocationIdentifier:
        return self.__monitoringLocationIdentifier

    @monitoringLocationIdentifier.setter
    def monitoringLocationIdentifier(self, val: MonitoringLocationIdentifier) -> None:
        self.__monitoringLocationIdentifier = (
            None if val is None else MonitoringLocationIdentifier(val)
        )

    @property
    def samplingComponentName(self) -> SamplingComponentName:
        return self.__samplingComponentName

    @samplingComponentName.setter
    def samplingComponentName(self, val: SamplingComponentName) -> None:
        self.__samplingComponentName = None if val is None else SamplingComponentName(val)

    @property
    def activityCommentText(self) -> CommentText:
        """
        General comments concerning the activity.
        """
        return self.__activityCommentText

    @activityCommentText.setter
    def activityCommentText(self, val: CommentText) -> None:
        """
        General comments concerning the activity.
        """
        self.__activityCommentText = None if val is None else CommentText(val)

    def generateXML(self, name: str = "ActivityDescription") -> str:  # noqa: C901
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__activityIdentifier is None:
                raise WQXException("Attribute 'activityIdentifier' is required.")
            line("ActivityIdentifier", self.__activityIdentifier)
            if self.__activityIdentifierUserSupplied is not None:
                line(
                    "ActivityIdentifierUserSupplied",
                    self.__activityIdentifierUserSupplied,
                )
            if self.__activityTypeCode is None:
                raise WQXException("Attribute 'activityTypeCode' is required.")
            line("ActivityTypeCode", self.__activityTypeCode)
            if self.__activityMediaName is None:
                raise WQXException("Attribute 'activityMediaName' is required.")
            line("ActivityMediaName", self.__activityMediaName)
            if self.__activityMediaSubdivisionName is not None:
                line("ActivityMediaSubdivisionName", self.__activityMediaSubdivisionName)
            if self.__activityStartDate is None:
                raise WQXException("Attribute 'activityStartDate' is required.")
            line("ActivityStartDate", str(self.__activityStartDate))
            if self.__activityStartTime is not None:
                asis(self.__activityStartTime.generateXML("ActivityStartTime"))
            if self.__activityEndDate is not None:
                line("ActivityEndDate", str(self.__activityEndDate))
            if self.__activityEndTime is not None:
                asis(self.__activityEndTime.generateXML("ActivityEndTime"))
            if self.__activityRelativeDepthName is not None:
                line("ActivityRelativeDepthName", self.__activityRelativeDepthName)
            if self.__activityDepthHeightMeasure is not None:
                asis(
                    self.__activityDepthHeightMeasure.generateXML(
                        "ActivityDepthHeightMeasure"
                    )
                )
            if self.__activityTopDepthHeightMeasure is not None:
                asis(
                    self.__activityTopDepthHeightMeasure.generateXML(
                        "ActivityTopDepthHeightMeasure"
                    )
                )
            if self.__activityBottomDepthHeightMeasure is not None:
                asis(
                    self.__activityBottomDepthHeightMeasure.generateXML(
                        "ActivityBottomDepthHeightMeasure"
                    )
                )
            if self.__activityDepthAltitudeReferencePointText is not None:
                line(
                    "ActivityDepthAltitudeReferencePointText",
                    self.__activityDepthAltitudeReferencePointText,
                )
            if self.__projectIdentifier is None or len(self.__projectIdentifier) < 1:
                raise WQXException(
                    "Attribute 'projectIdentifier' must be a list of 1 or more "
                    "ProjectIdentifier objects."
                )
            line("ProjectIdentifier", self.__projectIdentifier)
            for x in self.__activityConductingOrganizationText:
                line("ActivityConductingOrganizationText", x)
            if self.__monitoringLocationIdentifier is not None:
                line("MonitoringLocationIdentifier", self.__monitoringLocationIdentifier)
            if self.__samplingComponentName is not None:
                line("SamplingComponentName", self.__samplingComponentName)
            if self.__activityCommentText is not None:
                line("ActivityCommentText", self.__activityCommentText)

        return doc.getvalue()



from typing import List, Union

from yattag import Doc

from ..exceptions import WQXException
from .Entity_Update_Identifiers import UpdateIdentifiers


class WQXUpdateIdentifiers:
    """
    Main Schema used to update identifiers for major entities (projects, monitoring
    locations, activity, activity groups, and indexes).
    """

    __updateIdentifiers: List[UpdateIdentifiers]

    def __init__(
        self, o: dict = None, *, updateIdentifiers: List[UpdateIdentifiers] = None
    ):
        if isinstance(o, WQXUpdateIdentifiers):
            # Assign attributes from object without typechecking
            self.__updateIdentifiers = o.updateIdentifiers
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.updateIdentifiers = o.get("updateIdentifiers")
        else:
            # Assign attributes from named keywords with typechecking
            self.updateIdentifiers = updateIdentifiers

    @property
    def updateIdentifiers(self) -> List[UpdateIdentifiers]:
        return self.__updateIdentifiers

    @updateIdentifiers.setter
    def updateIdentifiers(
        self, val: Union[UpdateIdentifiers, List[UpdateIdentifiers]]
    ) -> None:
        if val is None:
            self.__updateIdentifiers = []
        elif isinstance(val, list):
            r: List[UpdateIdentifiers] = []
            for x in val:
                r.append(UpdateIdentifiers(x))
            self.__updateIdentifiers = r
        else:
            self.__updateIdentifiers = [UpdateIdentifiers(val)]

    def generateXML(self, name="UpdateIdentifiers") -> str:
        doc = Doc()
        asis = doc.asis
        tag = doc.tag

        with tag(
            name,
            ("xmlns", "http://www.exchangenetwork.net/schema/wqx/3"),
            ("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance"),
            (
                "xsi:schemaLocation",
                "http://www.exchangenetwork.net/schema/wqx/3 "
                "http://www.exchangenetwork.net/schema/wqx/3/index.xsd",
            ),
        ):
            if len(self.__updateIdentifiers) < 1:
                raise WQXException(
                    "Attribute 'updateIdentifiers' must be a list of 1 or more "
                    "UpdateIdentifiers objects."
                )
            for x in self.__updateIdentifiers:
                asis(x.generateXML("UpdateIdentifiers"))

        return doc.getvalue()



from yattag import Doc

from ..exceptions import WQXException
from .BibliographicReference import BibliographicReference
from .SimpleContent import (
    IndexTypeIdentifier,
    IndexTypeIdentifierContext,
    IndexTypeName,
    IndexTypeScaleText,
)


class IndexType:
    """
    This section identifies the index type reported as part of a biological or habitat
    index.
    """

    __indexTypeIdentifier: IndexTypeIdentifier
    __indexTypeIdentifierContext: IndexTypeIdentifierContext
    __indexTypeName: IndexTypeName
    __indexTypeCitation: BibliographicReference
    __indexTypeScaleText: IndexTypeScaleText

    def __init__(
        self,
        o: dict = None,
        *,
        indexTypeIdentifier: IndexTypeIdentifier = None,
        indexTypeIdentifierContext: IndexTypeIdentifierContext = None,
        indexTypeName: IndexTypeName = None,
        indexTypeCitation: BibliographicReference = None,
        indexTypeScaleText: IndexTypeScaleText = None
    ):
        if isinstance(o, IndexType):
            # Assign attributes from object without typechecking
            self.__indexTypeIdentifier = o.indexTypeIdentifier
            self.__indexTypeIdentifierContext = o.indexTypeIdentifierContext
            self.__indexTypeName = o.indexTypeName
            self.__indexTypeCitation = o.indexTypeCitation
            self.__indexTypeScaleText = o.indexTypeScaleText
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.indexTypeIdentifier = o.get("indexTypeIdentifier")
            self.indexTypeIdentifierContext = o.get("indexTypeIdentifierContext")
            self.indexTypeName = o.get("indexTypeName")
            self.indexTypeCitation = o.get("indexTypeCitation")
            self.indexTypeScaleText = o.get("indexTypeScaleText")
        else:
            # Assign attributes from named keywords with typechecking
            self.indexTypeIdentifier = indexTypeIdentifier
            self.indexTypeIdentifierContext = indexTypeIdentifierContext
            self.indexTypeName = indexTypeName
            self.indexTypeCitation = indexTypeCitation
            self.indexTypeScaleText = indexTypeScaleText

    @property
    def indexTypeIdentifier(self) -> IndexTypeIdentifier:
        return self.__indexTypeIdentifier

    @indexTypeIdentifier.setter
    def indexTypeIdentifier(self, val: IndexTypeIdentifier) -> None:
        self.__indexTypeIdentifier = IndexTypeIdentifier(val)

    @property
    def indexTypeIdentifierContext(self) -> IndexTypeIdentifierContext:
        return self.__indexTypeIdentifierContext

    @indexTypeIdentifierContext.setter
    def indexTypeIdentifierContext(self, val: IndexTypeIdentifierContext) -> None:
        self.__indexTypeIdentifierContext = IndexTypeIdentifierContext(val)

    @property
    def indexTypeName(self) -> IndexTypeName:
        return self.__indexTypeName

    @indexTypeName.setter
    def indexTypeName(self, val: IndexTypeName) -> None:
        self.__indexTypeName = IndexTypeName(val)

    @property
    def indexTypeCitation(self) -> BibliographicReference:
        """
        Provides additional description of the source that created or defined the index.
        """
        return self.__indexTypeCitation

    @indexTypeCitation.setter
    def indexTypeCitation(self, val: BibliographicReference) -> None:
        """
        Provides additional description of the source that created or defined the index.
        """
        self.__indexTypeCitation = None if val is None else BibliographicReference(val)

    @property
    def indexTypeScaleText(self) -> IndexTypeScaleText:
        return self.__indexTypeScaleText

    @indexTypeScaleText.setter
    def indexTypeScaleText(self, val: IndexTypeScaleText) -> None:
        self.__indexTypeScaleText = None if val is None else IndexTypeScaleText(val)

    def generateXML(self, name: str = "IndexType") -> str:
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__indexTypeIdentifier is None:
                raise WQXException("Attribute 'indexTypeIdentifier' is required.")
            line("IndexTypeIdentifier", self.__indexTypeIdentifier)
            if self.__indexTypeIdentifierContext is None:
                raise WQXException("Attribute 'indexTypeIdentifierContext' is required.")
            line("IndexTypeIdentifierContext", self.__indexTypeIdentifierContext)
            if self.__indexTypeName is None:
                raise WQXException("Attribute 'indexTypeName' is required.")
            line("IndexTypeName", self.__indexTypeName)
            if self.__indexTypeCitation is not None:
                asis(self.__indexTypeCitation.generateXML("IndexTypeCitation"))
            if self.__indexTypeScaleText is not None:
                line("IndexTypeScaleText", self.__indexTypeScaleText)

        return doc.getvalue()



from yattag import Doc

from ..exceptions import WQXException
from .SimpleContent import (
    MonitoringLocationIdentifier,
    MonitoringLocationIdentifierContext,
)


class AlternateMonitoringLocationIdentity:
    """
    Alternate identifications of a monitoring location.
    """

    __monitoringLocationIdentifier: MonitoringLocationIdentifier
    __monitoringLocationIdentifierContext: MonitoringLocationIdentifierContext

    def __init__(
        self,
        o: dict = None,
        *,
        monitoringLocationIdentifier: MonitoringLocationIdentifier = None,
        monitoringLocationIdentifierContext: MonitoringLocationIdentifierContext = None
    ):
        if isinstance(o, AlternateMonitoringLocationIdentity):
            # Assign attributes from object without typechecking
            self.__monitoringLocationIdentifier = o.monitoringLocationIdentifier
            self.__monitoringLocationIdentifierContext = (
                o.monitoringLocationIdentifierContext
            )
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.monitoringLocationIdentifier = o.get("monitoringLocationIdentifier")
            self.monitoringLocationIdentifierContext = o.get(
                "monitoringLocationIdentifierContext"
            )
        else:
            # Assign attributes from named keywords with typechecking
            self.monitoringLocationIdentifier = monitoringLocationIdentifier
            self.monitoringLocationIdentifierContext = monitoringLocationIdentifierContext

    @property
    def monitoringLocationIdentifier(self) -> MonitoringLocationIdentifier:
        return self.__monitoringLocationIdentifier

    @monitoringLocationIdentifier.setter
    def monitoringLocationIdentifier(self, val: MonitoringLocationIdentifier) -> None:
        self.__monitoringLocationIdentifier = MonitoringLocationIdentifier(val)

    @property
    def monitoringLocationIdentifierContext(self,) -> MonitoringLocationIdentifierContext:
        return self.__monitoringLocationIdentifierContext

    @monitoringLocationIdentifierContext.setter
    def monitoringLocationIdentifierContext(
        self, val: MonitoringLocationIdentifierContext
    ) -> None:
        self.__monitoringLocationIdentifierContext = MonitoringLocationIdentifierContext(
            val
        )

    def generateXML(self, name: str = "AlternateMonitoringLocationIdentity") -> str:
        doc = Doc()
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__monitoringLocationIdentifier is None:
                raise WQXException(
                    "Attribute 'MonitoringLocationIdentifier' is required."
                )
            line("MonitoringLocationIdentifier", self.__monitoringLocationIdentifier)
            if self.__monitoringLocationIdentifierContext is None:
                raise WQXException(
                    "Attribute 'MonitoringLocationIdentifierContext' is required."
                )
            line(
                "MonitoringLocationIdentifierContext",
                self.__monitoringLocationIdentifierContext,
            )

        return doc.getvalue()



from yattag import Doc

from ..exceptions import WQXException
from .SimpleContent import (
    FrequencyClassDescriptorCode,
    FrequencyClassDescriptorUnitCode,
    LowerClassBoundValue,
    UpperClassBoundValue,
)


class FrequencyClassInformation:
    """
    This section allows for the definition of a subgroup of biological communities by
    life stage, physical attribute, or abnormality to support frequency class studies.
    """

    __frequencyClassDescriptorCode: FrequencyClassDescriptorCode
    __frequencyClassDescriptorUnitCode: FrequencyClassDescriptorUnitCode
    __lowerClassBoundValue: LowerClassBoundValue
    __upperClassBoundValue: UpperClassBoundValue

    def __init__(
        self,
        o: dict = None,
        *,
        frequencyClassDescriptorCode: FrequencyClassDescriptorCode = None,
        frequencyClassDescriptorUnitCode: FrequencyClassDescriptorUnitCode = None,
        lowerClassBoundValue: LowerClassBoundValue = None,
        upperClassBoundValue: UpperClassBoundValue = None
    ):
        if isinstance(o, FrequencyClassInformation):
            # Assign attributes from object without typechecking
            self.__frequencyClassDescriptorCode = o.frequencyClassDescriptorCode
            self.__frequencyClassDescriptorUnitCode = o.frequencyClassDescriptorUnitCode
            self.__lowerClassBoundValue = o.lowerClassBoundValue
            self.__upperClassBoundValue = o.upperClassBoundValue
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.frequencyClassDescriptorCode = o.get("frequencyClassDescriptorCode")
            self.frequencyClassDescriptorUnitCode = o.get(
                "frequencyClassDescriptorUnitCode"
            )
            self.lowerClassBoundValue = o.get("lowerClassBoundValue")
            self.upperClassBoundValue = o.get("upperClassBoundValue")
        else:
            # Assign attributes from named keywords with typechecking
            self.frequencyClassDescriptorCode = frequencyClassDescriptorCode
            self.frequencyClassDescriptorUnitCode = frequencyClassDescriptorUnitCode
            self.lowerClassBoundValue = lowerClassBoundValue
            self.upperClassBoundValue = upperClassBoundValue

    @property
    def frequencyClassDescriptorCode(self) -> FrequencyClassDescriptorCode:
        return self.__frequencyClassDescriptorCode

    @frequencyClassDescriptorCode.setter
    def frequencyClassDescriptorCode(self, val: FrequencyClassDescriptorCode) -> None:
        self.__frequencyClassDescriptorCode = FrequencyClassDescriptorCode(val)

    @property
    def frequencyClassDescriptorUnitCode(self) -> FrequencyClassDescriptorUnitCode:
        return self.__frequencyClassDescriptorUnitCode

    @frequencyClassDescriptorUnitCode.setter
    def frequencyClassDescriptorUnitCode(
        self, val: FrequencyClassDescriptorUnitCode
    ) -> None:
        self.__frequencyClassDescriptorUnitCode = (
            None if val is None else FrequencyClassDescriptorUnitCode(val)
        )

    @property
    def lowerClassBoundValue(self) -> LowerClassBoundValue:
        return self.__lowerClassBoundValue

    @lowerClassBoundValue.setter
    def lowerClassBoundValue(self, val: LowerClassBoundValue) -> None:
        self.__lowerClassBoundValue = None if val is None else LowerClassBoundValue(val)

    @property
    def upperClassBoundValue(self) -> UpperClassBoundValue:
        return self.__upperClassBoundValue

    @upperClassBoundValue.setter
    def upperClassBoundValue(self, val: UpperClassBoundValue) -> None:
        self.__upperClassBoundValue = None if val is None else UpperClassBoundValue(val)

    def generateXML(self, name: str = "FrequencyClassInformation") -> str:
        doc = Doc()
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__frequencyClassDescriptorCode is None:
                raise WQXException(
                    "Attribute 'frequencyClassDescriptorCode' is required."
                )
            line("FrequencyClassDescriptorCode", self.__frequencyClassDescriptorCode)
            if self.__frequencyClassDescriptorUnitCode is not None:
                line(
                    "FrequencyClassDescriptorUnitCode",
                    self.__frequencyClassDescriptorUnitCode,
                )
            if self.__lowerClassBoundValue is not None:
                line("LowerClassBoundValue", self.__lowerClassBoundValue)
            if self.__upperClassBoundValue is not None:
                line("UpperClassBoundValue", self.__upperClassBoundValue)

        return doc.getvalue()
